@@ -27,8 +27,8 @@
 
 #include <config.h>
 #include <glib.h>
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <gconf/gconf-client.h>
 #include <bonobo.h>
 
 #include "Spell.h"
@@ -38,6 +38,7 @@ static BonoboObjectClass                  *dictionary_parent_class;
 static POA_GNOME_Spell_Dictionary__vepv    dictionary_vepv;
 
 #define DICT_DEBUG(x) x
+#define GNOME_SPELL_GCONF_DIR "/GNOME/Spell"
 
 static void release_engines (GNOMESpellDictionary *dict);
 
@@ -54,7 +55,7 @@ raise_error (CORBA_Environment * ev, const gchar *s)
 }
 
 static void
-dictionary_object_init (GtkObject *object)
+gnome_spell_dictionary_init (GObject *object)
 {
 	GNOMESpellDictionary *dict = GNOME_SPELL_DICTIONARY (object);
 
@@ -63,13 +64,13 @@ dictionary_object_init (GtkObject *object)
 }
 
 static void
-dictionary_object_destroy (GtkObject *object)
+dictionary_finalize (GObject *object)
 {
 	GNOMESpellDictionary *dictionary = GNOME_SPELL_DICTIONARY (object);
 
 	release_engines (dictionary);
 
-	GTK_OBJECT_CLASS (dictionary_parent_class)->destroy (object);
+	G_OBJECT_CLASS (dictionary_parent_class)->finalize (object);
 }
 
 static SpellEngine *
@@ -78,9 +79,9 @@ new_engine (const gchar *language)
 	SpellEngine *se;
 
 	se = g_new0 (SpellEngine, 1);
-	se->config = new_pspell_config ();
-	pspell_config_replace (se->config, "language-tag", language);
-	pspell_config_replace (se->config, "encoding", "utf-8");
+	se->config = new_aspell_config ();
+	aspell_config_replace (se->config, "language-tag", language);
+	aspell_config_replace (se->config, "encoding", "utf-8");
 	se->changed = TRUE;
 
 	return se;
@@ -92,10 +93,10 @@ release_engines (GNOMESpellDictionary *dict)
 	for (; dict->engines; ) {
 		SpellEngine *se = dict->engines->data;
 
-		if (se->manager)
-			delete_pspell_manager (se->manager);
+		if (se->speller)
+			delete_aspell_speller (se->speller);
 		if (se->config)
-			delete_pspell_config (se->config);
+			delete_aspell_config (se->config);
 		g_free (se);
 		dict->engines = g_slist_remove (dict->engines, se);
 	}
@@ -104,7 +105,7 @@ release_engines (GNOMESpellDictionary *dict)
 	dict->changed = TRUE;
 }
 
-#define KNOWN_LANGUAGES 22
+#define KNOWN_LANGUAGES 25
 static gchar *known_languages [KNOWN_LANGUAGES*2 + 1] = {
 	"br", N_("Breton"),
 	"ca", N_("Catalan"),
@@ -112,6 +113,7 @@ static gchar *known_languages [KNOWN_LANGUAGES*2 + 1] = {
 	"da", N_("Danish"),
 	"de_de", N_("German (Germany)"),
 	"de_ch", N_("German (Swiss)"),
+	"el", N_("Greek"),
 	"en_us", N_("English (American)"),
 	"en_gb", N_("English (British)"),
 	"en_ca", N_("English (Canadian)"),
@@ -127,7 +129,9 @@ static gchar *known_languages [KNOWN_LANGUAGES*2 + 1] = {
 	"pt_pt", N_("Portuguese (Portugal)"),
 	"pt_br", N_("Portuguese (Brazilian)"),
 	"ru", N_("Russian"),
+	"sk", N_("Slovak"),
 	"sv", N_("Swedish"),
+	"uk", N_("Ukrainian"),
 	NULL
 };
 
@@ -135,24 +139,24 @@ static GSList *
 get_languages_real (gint *ln)
 {
 	GSList *langs;
-	PspellCanHaveError *err;
-	PspellConfig  *config;
-	PspellManager *manager;
+	AspellCanHaveError *err;
+	AspellConfig  *config;
+	AspellSpeller *speller;
 	gint i;
 
-	/* printf ("get_languages_real\n"); */
+	DICT_DEBUG (printf ("get_languages_real\n"));
 
 	langs = NULL;
 	*ln = 0;
 	for (i=0; known_languages [i]; i++) {
-		config = new_pspell_config ();
-		pspell_config_replace (config, "language-tag", known_languages [i]);
+		config = new_aspell_config ();
+		aspell_config_replace (config, "language-tag", known_languages [i]);
 		i++;
-		err = new_pspell_manager (config);
-		if (pspell_error_number (err) == 0) {
-			manager = to_pspell_manager (err);
+		err = new_aspell_speller (config);
+		if (aspell_error_number (err) == 0) {
+			speller = to_aspell_speller (err);
 			DICT_DEBUG (printf ("Language: %s\n", known_languages [i]));
-			delete_pspell_manager (manager);
+			delete_aspell_speller (speller);
 			langs = g_slist_prepend (langs, GINT_TO_POINTER (i - 1));
 			(*ln) ++;
 		}
@@ -162,7 +166,7 @@ get_languages_real (gint *ln)
 }
 
 static GSList *
-get_languages_load (gint *ln)
+get_languages_load (GConfClient *gc, gint *ln)
 {
 	GString *str;
 	GSList *langs = NULL;
@@ -171,10 +175,10 @@ get_languages_load (gint *ln)
 	/* printf ("get_languages_load\n"); */
 
 	str = g_string_new (NULL);
-	*ln = gnome_config_get_int ("languages");
+	*ln = gconf_client_get_int (gc, GNOME_SPELL_GCONF_DIR "/languages", NULL);
 	for (i = 0; i < *ln; i++) {
-		g_string_sprintf (str, "language%d", i);
-		lang_num = gnome_config_get_int (str->str);
+		g_string_sprintf (str, GNOME_SPELL_GCONF_DIR "/language%d", i);
+		lang_num = gconf_client_get_int (gc, str->str, NULL);
 		langs = g_slist_prepend (langs, GINT_TO_POINTER (lang_num));
 	}
 
@@ -185,34 +189,36 @@ static GSList *
 get_languages (gint *ln)
 {
 	GSList *langs, *l;
+	GConfClient *gc;
 	time_t mtime;
 	struct stat buf;
 	gint i, kl;
 
-	gnome_config_push_prefix ("gnome-spell/Languages/");
-	mtime = gnome_config_get_int ("mtime=0");
-	kl = gnome_config_get_int ("known_languages=0");
+	gc = gconf_client_get_default ();
 
-	stat ("/opt/gnome/share" "/pspell", &buf);
-	if (buf.st_mtime != mtime || kl != KNOWN_LANGUAGES) {
+	mtime = gconf_client_get_int (gc, GNOME_SPELL_GCONF_DIR "/mtime", NULL);
+	kl = gconf_client_get_int (gc, GNOME_SPELL_GCONF_DIR "/known_languages", NULL);
+
+ 	if (stat (ASPELL_DICT, &buf) || buf.st_mtime != mtime || kl != KNOWN_LANGUAGES) {
 		GString *str;
 		langs = get_languages_real (ln);
 
 		str = g_string_new (NULL);
-		gnome_config_set_int ("languages", *ln);
+		gconf_client_set_int (gc, GNOME_SPELL_GCONF_DIR "/languages", *ln, NULL);
 		for (l = langs, i = 0; i < *ln; i ++) {
-			g_string_sprintf (str, "language%d", *ln - i - 1);
-			gnome_config_set_int (str->str, GPOINTER_TO_INT (l->data));
+			g_string_sprintf (str, GNOME_SPELL_GCONF_DIR "/language%d", *ln - i - 1);
+			gconf_client_set_int (gc, str->str, GPOINTER_TO_INT (l->data), NULL);
 			l = l->next;
 		}
-		gnome_config_set_int ("mtime", buf.st_mtime);
-		gnome_config_set_int ("known_languages", KNOWN_LANGUAGES);
+		gconf_client_set_int (gc, GNOME_SPELL_GCONF_DIR "/mtime", buf.st_mtime, NULL);
+		gconf_client_set_int (gc, GNOME_SPELL_GCONF_DIR "/known_languages", KNOWN_LANGUAGES, NULL);
 		g_string_free (str, TRUE);
 		gnome_config_sync ();
 	} else
-		langs = get_languages_load (ln);
+		langs = get_languages_load (gc, ln);
 
-	gnome_config_pop_prefix ();
+	gconf_client_suggest_sync (gc, NULL);
+	g_object_unref (gc);
 
 	return langs;
 }
@@ -289,27 +295,27 @@ engines_config_replace (GSList *engines, const gchar *key, const gchar *value)
 	for (; engines; engines = engines->next) {
 		SpellEngine *se = (SpellEngine *) engines->data;
 
-		pspell_config_replace (se->config, key, value);
+		aspell_config_replace (se->config, key, value);
 	}
 }
 
 static void
 update_engine (SpellEngine *se, CORBA_Environment * ev)
 {
-	PspellCanHaveError *err;
+	AspellCanHaveError *err;
 
-	DICT_DEBUG (printf ("Dictionary: creating new pspell manager\n"));
+	DICT_DEBUG (printf ("Dictionary: creating new aspell speller\n"));
 
 	if (se->changed) {
-		if (se->manager)
-			delete_pspell_manager (se->manager);
-		err = new_pspell_manager (se->config);
-		if (pspell_error_number (err) != 0) {
-			g_warning ("pspell error: %s\n", pspell_error_message (err));
-			se->manager = NULL;
-			raise_error (ev, pspell_error_message (err));
+		if (se->speller)
+			delete_aspell_speller (se->speller);
+		err = new_aspell_speller (se->config);
+		if (aspell_error_number (err) != 0) {
+			g_warning ("aspell error: %s\n", aspell_error_message (err));
+			se->speller = NULL;
+			raise_error (ev, aspell_error_message (err));
 		} else {
-			se->manager = to_pspell_manager (err);
+			se->speller = to_aspell_speller (err);
 			se->changed = FALSE;
 		}
 	}
@@ -335,16 +341,16 @@ static CORBA_boolean
 engine_check_word (SpellEngine *se, const gchar *word, CORBA_Environment *ev)
 {
 	CORBA_boolean result = CORBA_TRUE;
-	gint pspell_result;
+	gint aspell_result;
 
-	g_return_val_if_fail (se->manager, CORBA_TRUE);
+	g_return_val_if_fail (se->speller, CORBA_TRUE);
 
-	pspell_result = pspell_manager_check (se->manager, word, strlen (word));
-	if (pspell_result == 0)
+	aspell_result = aspell_speller_check (se->speller, word, strlen (word));
+	if (aspell_result == 0)
 		result = CORBA_FALSE;
-	if (pspell_result == -1) {
-		g_warning ("pspell error: %s\n", pspell_manager_error_message (se->manager));
-		raise_error (ev, pspell_manager_error_message (se->manager));
+	if (aspell_result == -1) {
+		g_warning ("aspell error: %s\n", aspell_speller_error_message (se->speller));
+		raise_error (ev, aspell_speller_error_message (se->speller));
 	}
 
 	return result;
@@ -356,7 +362,7 @@ impl_gnome_spell_dictionary_check_word (PortableServer_Servant servant, const CO
 	GNOMESpellDictionary *dict = GNOME_SPELL_DICTIONARY (bonobo_object_from_servant (servant));
 	CORBA_boolean result = CORBA_FALSE;
 	GSList *l;
-	gboolean valid_manager = FALSE;
+	gboolean valid_speller = FALSE;
 
 	g_return_val_if_fail (word, result);
 
@@ -365,15 +371,15 @@ impl_gnome_spell_dictionary_check_word (PortableServer_Servant servant, const CO
 
 	update_engines (dict, ev);
 	for (l = dict->engines; l; l = l->next) {
-		if (((SpellEngine *) l->data)->manager) {
-			valid_manager = TRUE;
+		if (((SpellEngine *) l->data)->speller) {
+			valid_speller = TRUE;
 			if (engine_check_word ((SpellEngine *) l->data, word, ev))
 				result = CORBA_TRUE;
 		}
 	}
 
-	if (!valid_manager) {
-		DICT_DEBUG (printf ("Dictionary check_word: %s --> 1 (not valid manager)\n", word));
+	if (!valid_speller) {
+		DICT_DEBUG (printf ("Dictionary check_word: %s --> 1 (not valid speller)\n", word));
 		return CORBA_TRUE;
 	}
 	DICT_DEBUG (printf ("Dictionary check_word: %s --> %d\n", word, result));
@@ -391,8 +397,8 @@ impl_gnome_spell_dictionary_add_word_to_session (PortableServer_Servant servant,
 	update_engines (dict, ev);
 	DICT_DEBUG (printf ("Dictionary add_word_to_session: %s\n", word));
 	for (l = dict->engines; l; l = l->next) {
-		if (((SpellEngine *) l->data)->manager)
-			pspell_manager_add_to_session (((SpellEngine *) l->data)->manager, word, strlen (word));
+		if (((SpellEngine *) l->data)->speller)
+			aspell_speller_add_to_session (((SpellEngine *) l->data)->speller, word, strlen (word));
 	}
 }
 
@@ -410,9 +416,9 @@ impl_gnome_spell_dictionary_add_word_to_personal (PortableServer_Servant servant
 	for (l = dict->engines; l; l = l->next) {
 		SpellEngine *se = (SpellEngine *) l->data;
 
-		if (se->manager) {
-			pspell_manager_add_to_personal (se->manager, word, strlen (word));
-			pspell_manager_save_all_word_lists (se->manager);
+		if (se->speller) {
+			aspell_speller_add_to_personal (se->speller, word, strlen (word));
+			aspell_speller_save_all_word_lists (se->speller);
 		}
 	}
 }
@@ -432,10 +438,10 @@ impl_gnome_spell_dictionary_set_correction (PortableServer_Servant servant,
 	for (l = dict->engines; l; l = l->next) {
 		SpellEngine *se = (SpellEngine *) l->data;
 
-		if (se->manager) {
-			pspell_manager_store_replacement (se->manager, word, strlen (word),
+		if (se->speller) {
+			aspell_speller_store_replacement (se->speller, word, strlen (word),
 							  replacement, strlen (replacement));
-			pspell_manager_save_all_word_lists (se->manager);
+			aspell_speller_save_all_word_lists (se->speller);
 		}
 	}
 }
@@ -445,8 +451,8 @@ impl_gnome_spell_dictionary_get_suggestions (PortableServer_Servant servant,
 					     const CORBA_char *word, CORBA_Environment *ev)
 {
 	GNOMESpellDictionary  *dict = GNOME_SPELL_DICTIONARY (bonobo_object_from_servant (servant));
-	const PspellWordList  *suggestions;
-	PspellStringEmulation *elements;
+	const AspellWordList  *suggestions;
+	AspellStringEnumeration *elements;
 	GNOME_Spell_StringSeq *seq = NULL;
 	GSList *l, *suggestion_list = NULL;
 	gint i, len, pos;
@@ -460,10 +466,10 @@ impl_gnome_spell_dictionary_get_suggestions (PortableServer_Servant servant,
 	for (l = dict->engines; l; l = l->next) {
 		SpellEngine *se = (SpellEngine *) l->data;
 
-		if (se->manager) {
-			suggestions  = pspell_manager_suggest (se->manager, word, strlen (word));
+		if (se->speller) {
+			suggestions  = aspell_speller_suggest (se->speller, word, strlen (word));
 			suggestion_list = g_slist_prepend (suggestion_list, (gpointer) suggestions);
-			len += pspell_word_list_size (suggestions);
+			len += aspell_word_list_size (suggestions);
 		}
 	}
 
@@ -479,12 +485,12 @@ impl_gnome_spell_dictionary_get_suggestions (PortableServer_Servant servant,
 	for (l = suggestion_list; l; l = l->next) {
 		gint list_len;
 
-		suggestions = (const PspellWordList  *) l->data;
-		elements = pspell_word_list_elements (suggestions);
-		list_len = pspell_word_list_size (suggestions);
+		suggestions = (const AspellWordList  *) l->data;
+		elements = aspell_word_list_elements (suggestions);
+		list_len = aspell_word_list_size (suggestions);
 		for (i = 0; i < list_len; i ++, pos ++)
-			seq->_buffer [pos] = CORBA_string_dup (pspell_string_emulation_next (elements));
-		delete_pspell_string_emulation (elements);
+			seq->_buffer [pos] = CORBA_string_dup (aspell_string_enumeration_next (elements));
+		delete_aspell_string_enumeration (elements);
 	}
 	CORBA_sequence_set_release (seq, CORBA_TRUE);
 	g_slist_free (suggestion_list);
@@ -503,6 +509,19 @@ gnome_spell_dictionary_get_epv (void)
 
 	epv = g_new0 (POA_GNOME_Spell_Dictionary__epv, 1);
 
+
+	return epv;
+}
+
+static void
+gnome_spell_dictionary_class_init (GNOMESpellDictionaryClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	POA_GNOME_Spell_Dictionary__epv *epv = &klass->epv;
+
+	dictionary_parent_class   = g_type_class_ref (BONOBO_TYPE_OBJECT);
+	object_class->finalize    = dictionary_finalize;
+
 	epv->getLanguages         = impl_gnome_spell_dictionary_get_languages;
 	epv->setLanguage          = impl_gnome_spell_dictionary_set_language;
 	epv->checkWord            = impl_gnome_spell_dictionary_check_word;
@@ -510,111 +529,16 @@ gnome_spell_dictionary_get_epv (void)
 	epv->addWordToPersonal    = impl_gnome_spell_dictionary_add_word_to_personal;
 	epv->getSuggestions       = impl_gnome_spell_dictionary_get_suggestions;
 	epv->setCorrection        = impl_gnome_spell_dictionary_set_correction;
-
-	return epv;
 }
 
-static void
-init_dictionary_corba_class (void)
-{
-	dictionary_vepv.Bonobo_Unknown_epv         = bonobo_object_get_epv ();
-	dictionary_vepv.GNOME_Spell_Dictionary_epv = gnome_spell_dictionary_get_epv ();
-}
-
-static void
-dictionary_class_init (GNOMESpellDictionaryClass *klass)
-{
-	GtkObjectClass *object_class = (GtkObjectClass *) klass;
-
-	dictionary_parent_class  = gtk_type_class (bonobo_object_get_type ());
-	object_class->destroy    = dictionary_object_destroy;
-
-	init_dictionary_corba_class ();
-}
-
-GtkType
-gnome_spell_dictionary_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"Dictionary",
-			sizeof (GNOMESpellDictionary),
-			sizeof (GNOMESpellDictionaryClass),
-			(GtkClassInitFunc) dictionary_class_init,
-			(GtkObjectInitFunc) dictionary_object_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (bonobo_object_get_type (), &info);
-	}
-
-	return type;
-}
-
-BonoboObject *
-dictionary_construct (BonoboObject *dictionary, GNOME_Spell_Dictionary corba_dictionary)
-{
-	CORBA_Environment ev;
-	static int i;
-	
-	g_return_val_if_fail (dictionary != NULL, NULL);
-	g_return_val_if_fail (IS_GNOME_SPELL_DICTIONARY (dictionary), NULL);
-	g_return_val_if_fail (corba_dictionary != CORBA_OBJECT_NIL, NULL);
-
-	if (!bonobo_object_construct (dictionary, (CORBA_Object) corba_dictionary))
-		return NULL;
-
-	CORBA_exception_init (&ev);
-	CORBA_exception_free (&ev);
-
-	return dictionary;
-}
-
-/*
- * This routine creates the ORBit CORBA server and initializes the
- * CORBA side of things
- */
-static GNOME_Spell_Dictionary
-create_dictionary (BonoboObject *dictionary)
-{
-	POA_GNOME_Spell_Dictionary *servant;
-	CORBA_Environment ev;
-
-	servant       = (POA_GNOME_Spell_Dictionary *) g_new0 (BonoboObjectServant, 1);
-	servant->vepv = &dictionary_vepv;
-
-	CORBA_exception_init (&ev);
-	POA_GNOME_Spell_Dictionary__init ((PortableServer_Servant) servant, &ev);
-	ORBIT_OBJECT_KEY (servant->_private)->object = NULL;
-
-	if (ev._major != CORBA_NO_EXCEPTION){
-		g_free (servant);
-		CORBA_exception_free (&ev);
-		return CORBA_OBJECT_NIL;
-	}
-
-	CORBA_exception_free (&ev);
-
-	return (GNOME_Spell_Dictionary) bonobo_object_activate_servant (dictionary, servant);
-}
+BONOBO_TYPE_FUNC_FULL (
+	GNOMESpellDictionary,          /* Glib class name */
+	GNOME_Spell_Dictionary,        /* CORBA interface name */
+	BONOBO_TYPE_OBJECT,            /* parent type */
+	gnome_spell_dictionary);       /* local prefix ie. 'echo'_class_init */
 
 BonoboObject *
 gnome_spell_dictionary_new (void)
 {
-	BonoboObject *dictionary;
-	GNOME_Spell_Dictionary corba_dictionary;
-
-              dictionary = gtk_type_new (gnome_spell_dictionary_get_type ());
-	corba_dictionary = create_dictionary (dictionary);
-
-	if (corba_dictionary == CORBA_OBJECT_NIL) {
-		bonobo_object_unref (BONOBO_OBJECT (dictionary));
-		return NULL;
-	}
-	
-	return dictionary_construct (dictionary, corba_dictionary);
+	return g_object_new (GNOME_SPELL_DICTIONARY_TYPE, NULL);
 }
